<?php

namespace Drupal\template_block\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a 'TemplateBlock' block plugin.
 *
 * A Template Block block has no content other than what is defined in a Twig
 * template file. The Template Block block's configuration form allows editors
 * to specify the name of the Twig template file to use.
 *
 * @Block(
 *  id = "template_block",
 *  admin_label = @Translation("Template Block"),
 *  description = @Translation("A block that places a Twig template."),
 *  category = @Translation("Template"),
 * )
 */
class TemplateBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function build() {
    return [
      '#theme' => 'template_block',
      '#template_suggestion' => $this->configuration['template_suggestion'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'template_suggestion' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['template_suggestion'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Template Suggestion'),
      '#description' => $this->t('Use only lowercase letters, numbers, and underscores, e.g. my_example_template.'),
      '#default_value' => $this->configuration['template_suggestion'],
      '#id' => 'template-block__suggestion-field',
      '#placeholder' => 'my_example_template',
      '#required' => TRUE,
    ];

    $form['template_name_preview'] = [
      '#type' => 'inline_template',
      '#template' => '<p><strong>Your template name:</strong> template-block--<span id="template-block__name-preview">my-example-template</span>.html.twig</p>',
      '#attached' => [
        'library' => ['template_block/name-preview'],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockValidate($form, FormStateInterface $form_state) {
    // Ensure the template suggestion contains only lowercase letters, numbers,
    // and underscores.
    if (!preg_match('/^[a-z0-9_]+$/', $form_state->getValue('template_suggestion'))) {
      $form_state->setErrorByName('template_suggestion', $this->t('The template suggestion must contain only lowercase letters, numbers, and underscores.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);

    $this->configuration['template_suggestion'] = $form_state->getValue('template_suggestion');
  }

}
